
# lib
LIB_FILE="/usr/share/lliurex/lliurex-netfuncs/lliurex-netfuncs.sh"
[ -e "$LIB_FILE" ] || exit 1
. ${LIB_FILE}

INTERFACES_FILE="/etc/network/interfaces"

srv_nic_list(){
   NETWORK_CARDS=""
   if [ $(count_network_cards) -gt 0 ] ; then
      for c in $(get_network_cards) ; do
         is_wireless_card "$c" || echo "$c"
      done
   fi
}

test_interfaces(){
   IFACE_NAME="$1"
   test_iface_static "$IFACE_NAME" "$INTERFACES_FILE" || test_iface_auto "$IFACE_NAME" "$INTERFACES_FILE" || return 1
   return 0
}

test_iface_dhcp(){
   grep -q "$(iface_dhcp_regexp "$1")" $2 || return 1
   return 0
}

test_iface_static(){
   grep -q "$(iface_static_regexp "$1")" $2 || return 1
   return 0
}

test_iface_auto(){
   grep -q "^[[:blank:]]*auto\b.*\b$1\b" $2 || return 1
   return 0

}

test_iface_address(){
   test_iface_option "$1" "$2" "address" || return 1
   return 0
}

test_iface_netmask(){
   test_iface_option "$1" "$2" "netmask" || return 1
   return 0
}

test_iface_gateway(){
   test_iface_option "$1" "$2" "gateway" || return 1
   return 0
}

test_iface_option(){
   [ "$(sed -ne "/$(iface_static_regexp "$1")/,/^[[:blank:]]*iface[[:blank:]]/{/^[[:blank:]]*$3\b/p}" "$2")" ] || return 1
   return 0
}

iface_dhcp_regexp(){
   echo "$(iface_regexp "$1")[[:blank:]]*dhcp\b"
}

iface_static_regexp(){
   echo "$(iface_regexp "$1")[[:blank:]]*static\b"
}

iface_regexp(){
   echo "^[[:blank:]]*iface[[:blank:]]\+$1[[:blank:]]\+inet\b"
}

test_nic(){
   [ "$1" ] || return 1
   srv_nic_list |grep -q "^${1}$" || return 1
   return 0
}

fix_interfaces(){
   IFACE_NAME="$1"
   TMP_FILE="$(tempfile)"
   CHANGES=""
   IP="$2"
   NETMASK="$3"
   GW="$4"
   [ "$IP" ] || IP="$(get_srv_ip default)"
   if [ "$IP" != "dhcp" ] ; then
      [ "$NETMASK" ] || NETMASK="$(get_srv_mask default)"
   fi

   [ ! -r "$INTERFACES_FILE" ] || cat "$INTERFACES_FILE" > "$TMP_FILE"
   if ! test_iface_auto "lo" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "auto lo" >> "$TMP_FILE"
   fi
   if ! grep -q "^[[:blank:]]*iface[[:blank:]]\+lo[[:blank:]]\+inet[[:blank:]]\+loopback\b" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "iface lo inet loopback" >> "$TMP_FILE"
   fi
   if ! test_iface_auto "$IFACE_NAME" "$TMP_FILE" ; then
      CHANGES="Y"
      echo "auto $IFACE_NAME" >> "$TMP_FILE"
   fi
   if [ "$IP" = "dhcp" ] ; then
      if ! test_iface_dhcp "$IFACE_NAME" "$TMP_FILE" ; then
         CHANGES="Y"
         sed -i -e "/$(iface_regexp "$IFACE_NAME")/d" "$TMP_FILE"
         echo "iface $IFACE_NAME inet dhcp" >> "$TMP_FILE"
      fi
   else
      if ! test_iface_static "$IFACE_NAME" "$TMP_FILE" ; then
         CHANGES="Y"
         sed -i -e "/$(iface_regexp "$IFACE_NAME")/d" "$TMP_FILE"
         echo "iface $IFACE_NAME inet static" >> "$TMP_FILE"
      fi
      if [ "$GW" ] && ! test_iface_gateway "$IFACE_NAME" "$TMP_FILE" ; then
         CHANGES="Y"
         sed -i -e "/$(iface_static_regexp "$IFACE_NAME")/agateway $GW" "$TMP_FILE"
      fi
      if ! test_iface_netmask "$IFACE_NAME" "$TMP_FILE" ; then
         CHANGES="Y"
         sed -i -e "/$(iface_static_regexp "$IFACE_NAME")/anetmask $NETMASK" "$TMP_FILE"
      fi
      if ! test_iface_address "$IFACE_NAME" "$TMP_FILE" ; then
         CHANGES="Y"
         sed -i -e "/$(iface_static_regexp "$IFACE_NAME")/aaddress $IP" "$TMP_FILE"
      fi
   fi
   if [ "$CHANGES" ] ; then
      TIMESTAMP_FILE="${INTERFACES_FILE}.$(date +%Y%m%d-%H%M%S)"
      cp -f "$INTERFACES_FILE" "$TIMESTAMP_FILE"
      llxcfg-install "$TMP_FILE" "$INTERFACES_FILE"
   fi
   rm -f "$TMP_FILE"
   return 0
}

set_interfaces(){
   [ "$1" ] || return 1
   TMP_FILE="$(tempfile)"
   sed -e "/$(iface_static_regexp "$1")/,/$(iface_regexp ".*")/{/^[[:blank:]]*auto\b/p;/$(iface_static_regexp "$1")/d;/$(iface_regexp ".*")/p;d}" "$INTERFACES_FILE" > "$TMP_FILE"
   llxcfg-install "$TMP_FILE" "$INTERFACES_FILE"
   rm -f "$TMP_FILE"
   fix_interfaces "$@"
   return 0
}

